/* When we are developing extensions and BlueJScript projects,
 * it is often important to be able to show the JavaScript console.
 * This adds a menu to BlueJ, so we can see any errors or output
 * generated by our extension
 */
// jsMenu.addAction ('Show Console',  'jsConsole.setVisible(true);'  );

/* Ahh. Languages with globals. How dangerous. In this case,
 * we're grabbing both our current location and the parent
 * frame; the latter is useful for when we want to display dialog
 * boxes to the user.
 */
htmlBase = constants.getParentUrl();
parentFrame = bluej.getCurrentFrame();
// The 'pane' gets a handle on the JOptionPane, which
// we will use for message boxes as well.
pane = Packages.javax.swing.JOptionPane;

/* JavaScript doesn't provide you with a lot of ways of organizing
 * your code; keeping things in separate files is one of the best
 * ways to keep yourself organized. In this case, we have a small
 * library that we load.
 */
load(new java.net.URL (htmlBase + '/library.js')); 


/* There are two ways we can display our HTML frame; one 
 * is inside of the BlueJ class viewer, and the other is as
 * a free-floating window. Here, we've chosen to make it a
 * free window, and have sized it to be taller than it is wide.
 * This may not work so well on monitors smaller than 640x480.
 */
//html.showInside(parentFrame);            
html.setSize(520,620);

/* Now, we load the first HTML page. 
 * From this point on, our interaction with the user
 * will be event-based. That is, we will just be listening
 * for events from the pane, and responding to them accordingly.
 */
html.load (htmlBase + '/index.html');


// There may be better ways to do this; we wanted to 
// track how many times a user tries to transition from
// page three to page four in our demo.
count34 = 0;

/* CONTRACT
 * transition :: int int -> void
 * PURPOSE
 * The transition function is called from every webpage; 
 * we expect that all URLs will be of the form:
 * <a href="javascript:transition(1,2)">
 * where the hyperlink explicitly calls this function with the 
 * target start and end pages. Using this information, we decide
 * if the conditions have been met to allow the user to continue
 * on in the tutorial.
 */
function transition(start, end) {

    ////////////////////
    // Node 1 --> Node 2
    if((start == 1) &&
       (end   == 2)) {
  
        if(hasConstructor("Cow", ["String", "String"])) {
            pane.showMessageDialog(parentFrame, getCongrat());
            html.load(htmlBase+"/add_method.html");
        } else {
            pane.showMessageDialog(parentFrame, 
                                   "I don't think you've added\n"
                                   + "a constructor yet.");
        }
  
        ////////////////////
        // Node 2 --> Node 3
    } else if ((start == 2) &&
               (end   == 3)) {
        pane.showMessageDialog(parentFrame, getCongrat());
        html.load(htmlBase+"/runtime_check.html");
  
        ////////////////
        // Node 3 --> Node 4
    } else if ((start == 3) &&
               (end   == 4)) {
        if(testCows()) {
            html.load(htmlBase+"/all_done.html");
        } else {
            if(count34 == 0) {
                pane.showMessageDialog(parentFrame, getProblem());
            } else if(count34 > 0) {
                pane.showMessageDialog(parentFrame,  
                                       "I think there's still something wrong.");
            } 
            count34++;
            html.load(htmlBase+"/check_cows.html");
        }
    }
    
    return true;
} 

/* CONTRACT
 * invokeSpeak :: string string -> boolean | string
 * PURPOSE
 * invokeSpeak creates a Cow object on the object bench in BlueJ, and
 * invokes it's speak method. If all goes well, we return what the cow says.
 * Certainly, there is more error checking that could be done in this method.
 */			   
function invokeSpeak(name, color) {
    bjPkg = bluej.getCurrentPackage();
    cowClass = bjPkg.getBClass("Cow");
    retval = false;
    if ( cowClass != null ) {
        // Grab the constructor
        cowConstructor = cowClass.getConstructor([convertType("String"), convertType("String")]);
        if (cowConstructor == null) { 
            //print ("cowConstructor : " + cowConstructor  + "\n"); 
            retval = false;
        }
    
        // Grab an object using this constructor
        cowObject   = cowConstructor.newInstance([name, color]);
        if (cowObject == null) { 
            //print ("cowObject : " + cowObject + "\n"); 
            retval = false;
        }
    
        cowObject.addToBench(name);    
        // Grab the method from the class
        speakMethod = cowClass.getDeclaredMethod("speak", null);
        if (speakMethod == null) { 
            //print("speakMethod : " + speakMethod  + "\n"); 
            retval = false;
        }
        // Invoke the method on the class
    
        message = speakMethod.invoke(cowObject, null);
        cowObject.removeFromBench();      
    }
  
    return message;
}

/* CONTRACT
 * testCow :: string string -> boolean
 * PURPOSE
 * This method wraps 'invokeSpeak', above, and checks to see if the
 * cow given says the "right thing". The "right thing" is described
 * to the user of the tutorial in the HTML documents displayed in BlueJ.
 * In this case, given a "purple" cow named "Bessie," it should say
 * "I'm Bessie, the talking purple cow"
 */
function testCow(name,color) {
    message = invokeSpeak(name,color);
  
    retval = false;
    if(message == "I'm " + name + ", the talking " + color + " cow") {
        retval = message;
    }
  
    return retval;
}

/* CONTRACT
 * testCowWithMessage :: string string -> void
 * PURPOSE
 * This does the same thing as "testCow", above, but displays an 
 * error dialog if things don't match up.
 * There is some seriously bad program design going on here
 * if we couldn't find a way to use "testCow" in our writing
 * of "testCowWithMessage."
 * 
 * Caveat functor. (Roughly translated, "programmer beware.")
 */
function testCowWithMessage(name, color) {
    message = invokeSpeak(name,color);
    
    if(!message.equals("I'm " + name + ", the talking " + color + " cow")) {
        pane.showMessageDialog(parentFrame, 
                               "I was expecting:" + "\n\"I'm " + name + ", the talking " 
                               + color + " cow\"\n\n" + "... but your cow said:" 
                               + "\n\"" + message + "\"\n");
    } else if (message.equals("I'm " + name + ", the talking " + color + " cow")) {
        pane.showMessageDialog(parentFrame,
                               "Looks like you got it!");
    } else {
        pane.showMessageDialog(parentFrame, 
                               "Looks like you need to keep trying.");
    }
  
  
}

/* CONTRACT
 * testCows :: void -> boolean
 * PURPOSE
 * Simplifies our life a bit; tests three cows, and returns
 * true if all of the cows say the right thing. 
 */
function testCows() {
    return (testCow("bessie", "red") &&
            testCow("margie", "green") &&
            testCow("frankie", "yellow"));
}


/* CONTRACT
 * getProblem :: void -> string
 * PURPOSE
 * Returns a string indicating to the user that some sort of 
 * problem exists in their code. Random.
 */
function getProblem(){
    messages = ["I think we've still got a problem.",
                "Houston, we have a problem.",
                "You're gonna have to fix something here...",
                "Something's not quite right..."];
    index = (Math.floor((Math.random() * messages.length)));
    return messages[index];
}

/* CONTRACT
 * getCongrat :: void -> string
 * PURPOSE
 * Returns a string indicating to the user that they've done
 * well. We'd like to point out how few programming environments
 * encourage success. 
 */
function getCongrat() {
    messages = ["Excellent!", "Woot!", "Well done!", "Nice work!",
                "Groovy.", "Far out, dude... far out."];
    index = (Math.floor((Math.random() * messages.length)));
    return messages[index];
}

function quit() {
    exit();
}
